library i_route;
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

class IRoute<R extends Route<dynamic>> extends RouteObserver<R> {
  static GlobalKey<NavigatorState> navigatorKey = GlobalKey();

  IRoute._();

  static IRoute get instance => IRoute._();

  @override
  void didPop(Route<dynamic> route, Route<dynamic>? previousRoute) {
    if (route is MaterialPageRouteLifecycle) {
      _RouterStack.instance.pop(targetWidget: route._widget);
    }
  }

  @override
  void didRemove(Route<dynamic> route, Route<dynamic>? previousRoute) {
    if (route is MaterialPageRouteLifecycle) {
      _RouterStack.instance.pop(targetWidget: route._widget);
    }
  }
  @override
  void didReplace({ Route<dynamic>? newRoute, Route<dynamic>? oldRoute }) {
    if (oldRoute is MaterialPageRouteLifecycle) {
      _RouterStack.instance.pop(targetWidget: oldRoute._widget);
    }
  }


  void show() => _RouterStack.instance.show();

  static Future<T?> pushNamed<T extends Object?>(routeName, {Object? arguments}) {
    return Navigator.push(navigatorKey.currentContext!, instance._routeNamed(routeName, arguments: arguments));
  }
  /// 仿照 [Navigator.popAndPushNamed] 的方法，先 pop 当前的页面，在 push 新页面。（会回调路由栈里的下一个页面的onResume和onPause方法）
  static Future<T?> popAndPushNamed<T extends Object?, TO extends Object?>(String routeName, {TO? result, Object? arguments}) {
    Navigator.pop<TO>(navigatorKey.currentContext!, result);
    return pushNamed(routeName, arguments: arguments);
  }

  /// 仿照 [Navigator.pushNamedAndRemoveUntil] 的方法，（会回调路由栈里的下一个页面的onResume和onPause方法）
  static Future<T?> pushNamedAndRemoveUntil<T extends Object?>(String newRouteName, RoutePredicate predicate, {Object? arguments}) {
    return Navigator.pushAndRemoveUntil(navigatorKey.currentContext!, instance._routeNamed(newRouteName, arguments: arguments), predicate);
  }
  /// 仿照 [Navigator.pushReplacementNamed] 的方法，（会回调路由栈里的下一个页面的onResume和onPause方法）
  static Future<T?> pushReplacementNamed<T extends Object?, TO extends Object?>(String routeName, {TO? result, Object? arguments}) {
    return Navigator.pushReplacement(navigatorKey.currentContext!, instance._routeNamed(routeName, arguments: arguments), result: result);
  }

  MaterialPageRoute<T> _routeNamed<T extends Object?>(routerName, {Object? arguments}) {
    var routeSettings = RouteSettings(name: routerName, arguments: arguments);
    MaterialPageRoute<T> materialPageRoute = ((navigatorKey.currentWidget as Navigator).onGenerateRoute!(routeSettings) as MaterialPageRoute<T>);
    MaterialPageRoute<T> route = MaterialPageRouteLifecycle(builder: materialPageRoute.builder, settings: routeSettings);
    return route;
  }
}

class _RouterStack {
  _RouterStack._();

  static _RouterStack get instance => _RouterStack._();
  static final List<Widget> _widgets = <Widget>[];

  void add(Widget widget) {
    _onPause(last);
    _widgets.add(widget);
    _onResume(widget);
  }

  Widget? pop({Widget? targetWidget}) {
    _onPause(targetWidget);
    if (_widgets.isEmpty) return null;
    if (targetWidget == null) {
      targetWidget = _widgets.removeLast();
      _onResume(last);
    } else {
      var lastIndexWhere = _widgets.lastIndexWhere((element) => element == targetWidget);
      if (lastIndexWhere < 0 || lastIndexWhere >= _widgets.length) return null;
      _widgets.removeAt(lastIndexWhere);
      if (lastIndexWhere == _widgets.length) {
        _onResume(last);
      }
    }
    return targetWidget;
  }

  void _onPause(widget) {
    if (widget != null && widget is RouterLifecycle) {
      (widget).onPause();
    }
  }

  void _onResume(widget) {
    if (widget != null && widget is RouterLifecycle) {
      (widget).onResume();
    }
  }

  Widget? get last => _widgets.isEmpty ? null : _widgets.last;

  void show() {
    showDialog(
        context: IRoute.navigatorKey.currentContext!,
        builder: (c) {
          return SimpleDialog(title: Text("路由栈"), children: _widgets.map((e) => SimpleDialogOption(child: Text(e.toString()))).toList().reversed.toList());
        });
    // _widgets.toString().log();
  }
}

mixin RouterLifecycle {
  void onResume();

  void onPause();
}

class MaterialPageRouteLifecycle<T> extends MaterialPageRoute<T> {
  MaterialPageRouteLifecycle({
    required WidgetBuilder builder,
    RouteSettings? settings,
    bool maintainState = true,
    bool fullscreenDialog = false,
  }) : super(
    builder: builder,
    settings: settings,
    maintainState: maintainState,
    fullscreenDialog: fullscreenDialog,
  );

  Widget? _widget;

  @override
  Widget buildContent(BuildContext context) {
    if (_widget == null) _widget = builder(context);
    if(_widget!=null){
      _RouterStack.instance.add(_widget!);
    }
    return _widget!;
  }

  Widget? get widget => _widget;
}
